# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"encode_file_as_utf8": "00-documentation.ipynb",
         "convert_nbs_to_md": "00-documentation.ipynb",
         "MyHTMLParser": "00-documentation.ipynb",
         "convert_df_to_md": "00-documentation.ipynb",
         "extract_div_to_md_table": "00-documentation.ipynb",
         "extract_div_to_md_tables": "00-documentation.ipynb",
         "clean_md_file_tables": "00-documentation.ipynb",
         "get_substring_idxs": "00-documentation.ipynb",
         "clean_md_file_img_fps": "00-documentation.ipynb",
         "get_ets_mkt_data": "01-mkt-prices.ipynb",
         "plot_long_term_avg": "01-mkt-prices.ipynb",
         "plot_ohlc_vol": "01-mkt-prices.ipynb",
         "plot_recent_ohlc_vol": "01-mkt-prices.ipynb",
         "get_country_raw_search": "02-eutl-installations.ipynb",
         "extract_search_df": "03-eutl-accounts.ipynb",
         "get_country_codes": "02-eutl-installations.ipynb",
         "get_installation_links_dataframe": "02-eutl-installations.ipynb",
         "get_url_root_and_params": "02-eutl-installations.ipynb",
         "get_num_pages": "02-eutl-installations.ipynb",
         "extract_installation_allocations_df": "02-eutl-installations.ipynb",
         "retry_request": "03-eutl-accounts.ipynb",
         "get_installation_allocations_df": "02-eutl-installations.ipynb",
         "get_all_installation_allocations_df": "02-eutl-installations.ipynb",
         "get_accounts_raw_search": "03-eutl-accounts.ipynb",
         "get_num_operating_accounts_pages": "03-eutl-accounts.ipynb",
         "get_full_search_df": "03-eutl-accounts.ipynb",
         "get_search_df": "03-eutl-accounts.ipynb",
         "account_id_to_url": "03-eutl-accounts.ipynb",
         "extract_key_table_soups": "03-eutl-accounts.ipynb",
         "extract_single_row_table_info": "03-eutl-accounts.ipynb",
         "extract_master_general_info": "03-eutl-accounts.ipynb",
         "extract_child_general_info": "03-eutl-accounts.ipynb",
         "extract_contact_info": "03-eutl-accounts.ipynb",
         "extract_address_info": "03-eutl-accounts.ipynb",
         "clean_dict_2nd_level_nulls": "03-eutl-accounts.ipynb",
         "extract_page_info": "03-eutl-accounts.ipynb",
         "collate_owner_info": "03-eutl-accounts.ipynb",
         "collate_unit_info": "03-eutl-accounts.ipynb",
         "construct_ets_unit_dfs": "03-eutl-accounts.ipynb",
         "constuct_da_ts_from_ts_dfs": "03-eutl-accounts.ipynb",
         "ts_dfs_to_separate_vars": "03-eutl-accounts.ipynb",
         "construct_installation_dfs": "03-eutl-accounts.ipynb",
         "get_installation_dfs": "03-eutl-accounts.ipynb",
         "construct_aircraft_dfs": "03-eutl-accounts.ipynb",
         "get_aircraft_dfs": "03-eutl-accounts.ipynb",
         "redownload_all_data": "03-eutl-accounts.ipynb",
         "app": "04-deployment.ipynb",
         "update_docs": "04-deployment.ipynb",
         "retrieve_all_data": "03-eutl-accounts.ipynb",
         "try_convert": "03-eutl-accounts.ipynb",
         "extract_compliance_df": "03-eutl-accounts.ipynb",
         "filter_for_year_indexes": "03-eutl-accounts.ipynb",
         "download_mkt_prices": "04-deployment.ipynb",
         "download_aircraft_accounts": "04-deployment.ipynb",
         "download_installation_accounts": "04-deployment.ipynb",
         "download_all_accounts_data": "04-deployment.ipynb"}

modules = ["documentation.py",
           "prices.py",
           "installations.py",
           "eutl.py",
           "accounts.py",
           "eutl/installations.py",
           "eutl/accounts.py",
           "deployment.py",
           "eutl/accounts.py",
           "eutl/accounts.py",
           "eutl/accounts.py",
           "eutl/accounts.py",
           "eutl/accounts.py",
           "eutl/accounts.py"]

doc_url = "https://OSUKED.github.io/ETS-Watch/"

git_url = "https://github.com/OSUKED/ETS-Watch"

def custom_doc_links(name): return None
